/* grecs - Gray's Extensible Configuration System
   Copyright (C) 2007-2011 Sergey Poznyakoff

   Grecs is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 3 of the License, or (at your
   option) any later version.

   Grecs is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with Grecs. If not, see <http://www.gnu.org/licenses/>. */

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif
#include <grecs.h>
#include <string.h>
#include <ctype.h>

void
grecs_version(struct grecs_version_info *pv)
{
	char *p;
	memset(pv, 0, sizeof(*pv));
	pv->package = PACKAGE_NAME;
	pv->version = PACKAGE_VERSION;
	pv->major = strtoul(pv->version, &p, 10);
	if (*p && *p == '.') {
		pv->minor = strtoul(p + 1, &p, 10);
		if (*p && *p == '.') {
			char *q;
			
			pv->patch = strtoul(p + 1, &q, 10);
			if (q == p + 1)
				pv->patch = 0;
			else
				p = q;
		}
	}
	pv->suffix = p;
}

int
grecs_version_cmp(const char *vstr)
{
	struct grecs_version_info v;
	char *p;
	unsigned long n;
	size_t len;
	int check;
	
	if (!vstr)
		return -1;
	grecs_version(&v);
	len = strcspn(vstr, " \t");
	if (vstr[len]) {
		if (len != strlen(v.package) || memcmp(v.package, vstr, len))
			return 1;
		for (vstr += len; *vstr && isspace(*vstr); vstr++)
			;
	}
	if (!*vstr)
		return 1;
	n = strtoul(vstr, &p, 10);
	if (n > v.major)
		return 1;
	check = n == v.major;
	if (*p && *p == '.') {
		n = strtoul(p + 1, &p, 10);
		if (check && n > v.minor)
			return 1;
		check = n == v.minor;
		if (*p && *p == '.') {
			n = strtoul(p + 1, &p, 10);
			if (check && n > v.patch)
				return 1;
		}
	}
	if (*p && (!v.suffix || strcmp(p, v.suffix)))
		return 1;
	return 0;
}
